﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel.Composition;
using System.Diagnostics;
using Microsoft.FSharp.Core;
using MoNo;
using MoNo.Studio;
using MoNo.CSharpUtils;

namespace QuickCsTemplate
{
  [Export( typeof( MoNo.Studio.IMenuFactory ) )]
  class AsyncMenuFactory : MenuFactoryForCSharp<QuickAppContext>
  {
    static MoNo.Progress<Unit> CalcProgressive1()
    {
      return Progress.ByAction( token =>
      {
        Trace.Write( "CalcProgressive1: " );
        for (int i = 0; i < 10; ++i ) {
          System.Threading.Thread.Sleep( 200 );
          Trace.Write( "." );
          token.Notify( 0.1 );
        }
        Trace.WriteLine( "done." );
      } );
    }

    static MoNo.Progress<double> CalcProgressive2()
    {
      return Progress.ByFunc( token =>
      {
        token.Run( 0.6, CalcProgressive1() );
        System.Threading.Thread.Sleep( 500 );
        Trace.WriteLine( "CalcProgressive2 - 1" );
        token.Notify( 0.2 );
        System.Threading.Thread.Sleep( 500 );
        Trace.WriteLine( "CalcProgressive2 - 2" );
        token.Notify( 0.2 );
        return Math.PI;
      } );
    }


    public override IEnumerable<MenuItem> EnumerateMenuItems( QuickAppContext con )
    {
      yield return MenuItemFactory.Folder( "Samples",
        MenuItemFactory.Folder( "Async/Progress Samples",
          con.CreateCommand( () =>
            con.ByAsync( token => {
              for (int i = 0; i < 10; ++i ) {
                Trace.WriteLine( "calculating ..." );
                System.Threading.Thread.Sleep( 300 );
                token.ThrowIfCancellationRequested();
              }
              Trace.WriteLine( "done." );
              return 1234;
            } ).ContinueWith( value => con.ShowMessage( string.Format( "result = {0}", value ) ) )
          ).ToMenuItem( "Async" ),

          con.CreateCommand( () =>
            con.ByAsync( async token =>
            {
              for ( int i = 0; i < 10; ++i ) {
                Trace.WriteLine( "calculating ..." );
                await Task.Delay( 300 );
                token.ThrowIfCancellationRequested();
              }
              Trace.WriteLine( "done." );
              return 1234;
            } ).ContinueWith( value => con.ShowMessage( string.Format( "result = {0}", value ) ) )
          ).ToMenuItem( "Async2" ),

          con.ByProgress( CalcProgressive2() )
          .ContinueWith( value => con.ShowMessage( string.Format( "result = {0}", value ) ) )
          .ToWpfCommand( con )
          .ToMenuItem( "Progress" )
        ) );
    }
  }
}
