﻿namespace QuickTemplate
open System
open System.ComponentModel.Composition
open System.Windows
open MoNo
open MoNo.Studio
open MoNo.Ctrl


type [<Export (typeof<Studio.IMenuFactory>)>] private AsyncMenuFactory() =
  inherit Studio.MenuFactory<QuickAppContext>()

  let trace     s = Diagnostics.Trace.Write s
  let traceLine s = Diagnostics.Trace.WriteLine s

  let calcProgressive1 () =
    Progress.Make (fun token ->
      trace "calcProgressive1: "
      for i = 1 to 10 do
        Threading.Thread.Sleep 200
        trace "."
        token.Notify 0.1
      traceLine "done.")

  let calcProgressive2 () =
    Progress.Make (fun token ->
      token.Run (0.6, calcProgressive1 ())
      Threading.Thread.Sleep 500
      traceLine "calcProgressive2 - 1"
      token.Notify 0.2
      Threading.Thread.Sleep 500
      traceLine "calcProgressive2 - 2"
      token.Notify 0.2
      Math.PI )

  override __.CreateMenuItems con =
    let items =
      [
        MenuItem.New ("Async",
          cont {
            let! result = con.ByAsync (async {
                for i = 1 to 10 do
                  traceLine "calculating ..."
                  Threading.Thread.Sleep 300
                traceLine "done."
                return 1234
              })
            do! con.Messenger.Send (Wpf.InformationMessage (sprintf "result = %d" result, "Async Calculation"))
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("Progress",
          cont {
            let! result = con.ByProgress (calcProgressive2 ())
            do! con.Messenger.Send (Wpf.InformationMessage (sprintf "result = %f" result, "Async Calculation"))
          } |> Wpf.Cont.toWpfCommand con None)
      ]
    [ MenuItem.Folder ("Samples", [ MenuItem.Folder ("Async/Progress Samples", items) ]) ]

