﻿namespace QuickTemplate
open System
open System.ComponentModel.Composition
open System.Windows
open MoNo
open MoNo.Studio


type [<Export (typeof<Studio.IMenuFactory>)>] private MessageMenuFactory() =
  inherit Studio.MenuFactory<QuickAppContext>()

  override __.CreateMenuItems con =
    let items =
      [
        // ViewModel 側で直接メッセージボックスを表示するコードは書けないため、Messenger 機能を利用して View 側にメッセージを送信

        MenuItem.New ("InformationMessage",
          cont {
            do! con.Messenger.Send (Wpf.InformationMessage ("Hello World", "InformationMessage", MessageBoxImage.Information))
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("OKCancelMessage",
          cont {
            let! ok = con.Messenger.Send (Wpf.OKCancelMessage ("OK? or Cancel?", "OKCancelMessage", MessageBoxImage.Exclamation))
            do! con.Messenger.Send (Wpf.InformationMessage ((if ok then "OK clicked" else "Cancel clicked"), "InformationMessage"))
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("YesNoCancelMessage",
          cont {
            let! yes = con.Messenger.Send (Wpf.YesNoCancelMessage ("Yes/No? or Cancel?", "YesNoCancelMessage", MessageBoxImage.Question))
            let response =
              match yes with
              | Some true   -> "Yes clicked"
              | Some false  -> "No clicked"
              | None        -> "Cancel clicked"
            do! con.Messenger.Send (Wpf.InformationMessage (response, "response"))
          } |> Wpf.Cont.toWpfCommand con None)


        // Messenger の機能をラップした ShowMesssage/GetOKCancel/GetYesNoCancel で簡単にメッセージ表示

        MenuItem.New ("ShowMessage",
          cont {
            do! con.ShowMessage "Hello ShowMessage(string)"
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("GetOKCancel",
          cont {
            let! ok = con.ShowOKCancelDialog "OK? or Cancel?"
            do! con.ShowMessage (if ok then "OK clicked" else "Cancel clicked")
          } |> Wpf.Cont.toWpfCommand con None)

        MenuItem.New ("GetYesNoCancel",
          cont {
            let! yes = con.ShowYesNoCancelDialog "Yes/No? or Cancel?"
            let response =
              match yes with
              | Some true   -> "Yes clicked"
              | Some false  -> "No clicked"
              | None        -> "Cancel clicked"
            do! con.ShowMessage response
          } |> Wpf.Cont.toWpfCommand con None)

      ]
    [ MenuItem.Folder ("Samples", [ MenuItem.Folder ("MessageBox Samples", items) ]) ]
