﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Input;
using Microsoft.FSharp.Core;
using Microsoft.FSharp.Collections;
using MoNo;
using MoNo.Geometries;
using MoNo.Studio;
using MoNo.CSharpUtils;

namespace StandardCsTemplate.ViewModels
{
  public class AppContext : MoNo.Studio.StudioAppContextForCSharp<AppModel>
  {
    public MenuItem[] MainMenuItems { get; private set; }
    public MenuItem[] MainToolBarItems { get; private set; }

    public AppContext() : base(
      new AppModel(),
      "MoNo.Studio 3D Standard Template",
      "アプリケーション固有ファイル形式 (*.myapp)|*.myapp|All files (*.*)|*.*" )
    {
      var importMenu = MenuItemFactory.Command(
        "Import", this.CreateCommand( () =>
          this.ShowOpenFileDialog( "STL files (*.stl)|*.stl|All files (*.*)|*.*" )
          .ContinueWith( path => this.ByProgress( this.App.ImportSTL( path ) ) )
          .Do( () => { CoreUT.Build(); this.RunFitAnimation(); } ) ),
        icon: "images/import.png" );

      var showSampleModalDialogMenu = MenuItemFactory.Command(
        "Show Sample Modal Dialog", this.CreateCommand( () =>
          this.ShowDialog( new VmSampleDialog( this ), MoNo.Wpf.DialogLocations.Center, true )
          .ContinueWith( vm => this.ShowMessage( string.Format( "Input Text = {0}", vm.InputText.Val ) ) )
        ) );

      var showSampleModalDialogMenu2 =
        this.CreateCommand( () =>
          from vm in this.ShowDialog( new VmSampleDialog( this ), MoNo.Wpf.DialogLocations.Center, true )
          from _ in this.ShowMessage( string.Format( "Input Text = {0}", vm.InputText.Val ) )
          select (Unit)null
        ).ToMenuItem( "Show Sample Modal Dialog 2" );

      var showSampleModelessDialogMenu = MenuItemFactory.Command(
        "Show Sample Modeless Dialog", this.CreateCommand( () =>
          this.ShowDialog( new VmSampleDialog( this ), MoNo.Wpf.DialogLocations.Center, false )
          .ContinueWith( vm => this.ShowMessage( string.Format( "Input Text = {0}", vm.InputText.Val ) ) )
        ) );

      this.MainMenuItems = new[]{
        MenuItemFactory.Folder( "File", base.ClearMenu, base.OpenMenu, base.SaveMenu, MenuItem.Separator, importMenu ),
        MenuItemFactory.Folder( "Edit", base.UndoMenu, base.RedoMenu, MenuItem.Separator, base.AbortMenu ),
        MenuItemFactory.Folder( "View", base.ViewFitMenu, base.ViewAlignMenu, base.FocusClipMenu, base.ShowEdgeMenu, MenuItem.Separator, base.ViewSettingMenu ),
        MenuItemFactory.Folder( "Debug", base.DebugMenuList ),
        MenuItemFactory.Folder( "Samples",
          MenuItemFactory.Command( "Console.WriteLine(\"Hello World\")", this.CreateCommand( () => Console.WriteLine( "Hello World" ) ) ),
          MenuItemFactory.Command( "Show MessageBox", this.CreateCommand( () => this.ShowMessage( "Hello World" ) ) ),
          showSampleModalDialogMenu,
          showSampleModalDialogMenu2,
          showSampleModelessDialogMenu
        )
      };

      this.MainToolBarItems = new[]{
        base.ClearMenu, base.OpenMenu, importMenu, base.SaveMenu, MenuItem.Separator,
        base.UndoMenu, base.RedoMenu, base.AbortMenu, MenuItem.Separator,
        base.ViewFitMenu, base.ViewAlignMenu, base.FocusClipMenu, base.ShowEdgeMenu
      };
    }

    public override IEnumerable<MenuItem> EnumerateContextMenuItems( MoNo.Wpf.Entry[] entries )
    {
      if ( entries.Length == 1 ) {
        var mesh = entries[0].Object as Mesh<Point3d>;
        if (mesh != null) {
          yield return
            MenuItemFactory.Command( "Show triangle count",
              this.CreateCommand( () =>
                this.ShowMessage( string.Format( "Facets count = {0}\nVertices count = {1}", mesh.Facets.Length, mesh.Vertices.Length ) )
              ) );
        }
      }
    }
  }
}
