﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel.Composition;
using Microsoft.FSharp.Core;
using MoNo;
using MoNo.Geometries;
using MoNo.IO;

namespace StandardCsTemplate
{
  public class Document : MoNo.Core.IMemorable
  {
    public ReactiveMemorable<Mesh<Point3d>> Mesh { get; private set; }

    public Document()
    {
      this.Mesh = new ReactiveMemorable<Mesh<Point3d>>( MoNo.Geometries.Mesh.empty<Point3d>() );
    }

    void MoNo.Core.IMemorable.AddRef() { MoNo.Core.History.AddRef( this.Mesh ); }
    void MoNo.Core.IMemorable.Release() { MoNo.Core.History.Release( this.Mesh ); }

    [Export( typeof( ISerializer ) )]
    class Serializer : AbstractSerializer<Document>
    {
      public override Guid TargetGuid
      {
        get { return new Guid( "f30452a6-fdc5-48be-9c78-b71adb1358d7" ); } // これはサンプルの暫定GUIDです。新しく生成したGUIDを設定して下さい。
      }

      protected override IEnumerable<object> GetDependencies( Document target )
      {
        yield return target.Mesh.Val;
      }

      protected override void WriteTarget( System.IO.BinaryWriter writer, Document target, IWritingSession session )
      {
        writer.Write( session.GetKey( target.Mesh.Val ) );
      }

      protected override Document ReadTarget( System.IO.BinaryReader reader, IReadingSession session )
      {
        var doc = new Document();
        doc.Mesh.Val = session.GetObject( reader.ReadInt32() ) as Mesh<Point3d>;
        return doc;
      }
    }
  }
}
