﻿namespace StandardTemplate
open System
open System.ComponentModel.Composition
open MoNo
open MoNo.Geometries
open MoNo.IO

type Document = {
    Mesh : Mesh3d
  }

type [<Export (typeof<ISerializer>)>] private DocumentSerializer () =
  inherit AbstractSerializer<Document> ()
  override val TargetGuid = Guid "b9e2023f-f48e-4a28-98a0-bbbf770dbf6c" // このGUIDはアプリケーションごとに新しく生成してください
  override __.GetDependencies (target : Document) = seq { yield target.Mesh :> obj }
  override __.WriteTarget (writer, target, session) = writer.Write (session.GetKey target.Mesh)
  override __.ReadTarget (reader, session) = { Mesh = reader.ReadInt32() |> session.GetObject :?> _ }


type AppModel () =
  inherit Studio.StudioAppModel<Document> (fun () -> { Mesh = Mesh.empty })
  member this.ImportSTL (path : string) =
    Progress.Make (fun token ->
      let tris = token.Run (0.5, STLFormat.readTris3d true path)
      let mesh = token.Run (0.5, Mesh.ofTris3dProgressive tris)
      mesh)
