﻿namespace StandardTemplate.ViewModels
open StandardTemplate
open System
open System.Windows.Input
open MoNo
open MoNo.Wpf
open MoNo.Ctrl
open MoNo.Studio

type AppContext() as con =
  inherit StudioAppContext<AppModel>(
    AppModel(),
    "MoNo.Studio 3D Standard Template",
    "アプリケーション固有ファイル形式 (*.myapp)|*.myapp|All files (*.*)|*.*")

  let importAction path = cont {
    let! mesh = con.ByProgress (con.App.ImportSTL path)
    con.App.Document.Val <- { Mesh = mesh }
    con.RunFitAnimation()
  }
  let importMenu =
    MenuItem.New ("Import",
      con.CreateCommand (cont {
        let! path = con.ShowOpenFileDialog "STL files (*.stl)|*.stl|All files (*.*)|*.*"
        do! importAction path
      }), icon = "images/import.png")

  member val MainMenuItems =
    [
      MenuItem.Folder ("File", [base.ClearMenu; base.OpenMenu; base.SaveMenu; Separator; importMenu])
      MenuItem.Folder ("Edit", [base.UndoMenu; base.RedoMenu; Separator; base.AbortMenu])
      MenuItem.Folder ("View", [base.ViewFitMenu; base.ViewAlignMenu; base.FocusClipMenu; base.ShowEdgeMenu; Separator; base.ViewSettingMenu])
      MenuItem.Folder ("Debug", base.DebugMenuList)
    ]

  member val MainToolBarItems =
    [
      base.ClearMenu; base.OpenMenu; importMenu; base.SaveMenu; Separator
      base.UndoMenu; base.RedoMenu; base.AbortMenu; Separator
      base.ViewFitMenu; base.ViewAlignMenu; base.FocusClipMenu
      base.ShowEdgeMenu; Separator

      // !!! CAUTION !!!  MoNo.Studio.Views.exe 以外のアセンブリに格納されているアイコンを使うときは下記の書式でアセンブリ名を書く必要がある
      MenuItem.New ("Hello",
        con.CreateCommand (cont {
          DebugTrace.put { Wpf.EntrySource.create (Point3d (-0.5, -0.5, Random().NextDouble())) with PointSize = Wpf.PropertySource.just 3.0f; Color = Wpf.PropertySource.just Windows.Media.Colors.Yellow }
          do! con.ShowMessage "Hello!"
        }),
        icon = "/StandardTemplate.Views;component/images/MyIcon.png")
    ]

  override con.GetContextMenuItems entries =
    if entries.Length = 1 then
      match entries.[0].Object with
      | :? Geometries.Tris3d as tris ->
        [
          MenuItem.New ("Show triangle count",
            con.CreateCommand (cont {
              do! con.ShowMessage (sprintf "Tris3d's triangle count = %d" tris.Items.Length)
            }))
        ]
      | :? Geometries.Mesh3d as mesh ->
        [
          MenuItem.New ("Show mesh properties",
            con.CreateCommand (cont {
              do! con.ShowMessage (sprintf "Facets count = %d\nVertices count = %d" mesh.Facets.Length mesh.Vertices.Length)
            }))
        ]
      | _ -> []
    else []

  override con.OnFileDrop files =
    cont {
      if files = null || files.Length <> 1 then
        do! con.ShowErrorMessage "ファイルは1つだけドラッグ・アンド・ドロップできます。"
        return! Cont.cancel()
      if files.[0].EndsWith (".myapp", StringComparison.CurrentCultureIgnoreCase) then
        do! con.ConfirmModificationAction
        do! con.OpenAction files.[0]
      elif files.[0].EndsWith (".stl", StringComparison.CurrentCultureIgnoreCase) then
        do! importAction files.[0]
    }
    |> Cont.executeBy con false